/******************************************************************************
* D_SPECVD.G
* Converts the DSGE parameters into unconditional moments
* Spectral density for selected observables
*
* upadted: 04/20/2006
******************************************************************************/
new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  __output = 1;
  _fcmptol = 1E-10;
  _verbose = 1;           /** 1 for verbose **/

#include dsge_mod.src;
#include dsge_para.src;
#include dsge_spec.g;
#include pathspec.g;

/********************************************************************
* Load Parameter Draws from MH Output
********************************************************************/
mhrun  = "1";
nblock = 10000;    /* block size */
nuse   = 1000;     /* use every nuse observation */

lpath  = parapath $+ "\\mhrun" $+ mhrun;
opath  = resupath $+ "\\mhrun" $+ mhrun;
chkdir = chk_dir(opath);

lpara       = lpath $+ "\\pa";
open fhpara = ^lpara for read;

/** initialize output files **/
ospec     = opath $+ "\\vdspec.dat";
create fhspec = ^ospec with SPEC, nvar^2, 8;

/** specify grid for omega **/
/** for business cycle frequencies use 0.0625*pi to 0.33*pi **/

om_min   = 0.0625*pi;
om_max   = 0.33*pi;
om_n     = 101;
om_step  = (om_max-om_min)/(om_n-1);
om_grid  = seqa(om_min,om_step,om_n);

/********************************************************************
* Loop
********************************************************************/
/** initialization **/
eofloop = 0;
loopct  = 1;

if _verbose;
    "";
    "Calculating moments...";
    "----------------------";
    pause(1e-20);
endif;

do until eofloop;

    if _verbose;
        "";
        "Block           " ftos(loopct,"%*.*lf",6,0);
        pause(1e-20);
    endif;

    /** read blocks of size nblock **/
    parasim = readr(fhpara,nblock);
    nread   = rows(parasim);

    /** use every nuse-th draw **/
    indseq  = seqa(1,1,nread);
    indseq  = indseq % nuse;
    indseq  = indseq .NE 0;

    parasim = delif(parasim,indseq);
    nread   = rows(parasim);

    if ismiss(parasim) == 0;

        vdsim = zeros(nread,nvar^2);

        j = 1;
        do until j > nread;

            para = parasim[j,.]';
            
            /* Solve the Model
            */
            {TT,QQ,RR,HH,DD,ZZ,VV,RC,QQ2}=sysmat(para,msel);
            
            /* manual adjustment: turn inverse velocity into real money
            ** balances:
            */
            ZZ[4,19]=0;
            
            
            
            sig_chol = sqrt(QQ);
            vdspec   = zeros(nvar,cols(QQ));
            varspec  = zeros(nvar,1);
            
            /* Compute Spectrum Shock by Shock
            */
            i = 1;
            do until i > cols(QQ);
            
               QQi = sig_chol[.,i]*sig_chol[.,i]';
               spec_mat_i  = dsge_spectrum(TT,RR,ZZ,QQi,om_grid);
               vdspec[.,i] = real( om_step*sumc(spec_mat_i[1:rows(spec_mat_i)-1,.]) );
               varspec = varspec + vdspec[.,i];
               
               i = i+1;
               
            endo;
            vdspec = vdspec ./ varspec;
            vdsim[j,.] =  vec(vdspec')';

            
            /* as a benchmark, compute unconditional variance
            */
            /* varallfreq =  diag( ZZ* real(dlyap(TT,RR*QQ*RR')) *ZZ' ); */
                                   
            if _verbose;
                if fmod(j,nread/10)==0; ".";; pause(1e-20);endif;
            endif;
            j = j+1;

        endo;

        /** write Output **/
        wr = writer(fhspec,vdsim);

    endif;

    if _verbose;
        "";
        "Draws Processed " ftos(rows(vdsim),"%*.*lf",6,0);
        "";pause(1e-20);
    endif;

    loopct = loopct + 1;
    eofloop = eof(fhpara);
     
endo;

closeall fhpara, fhspec;

end;

/********************************************************************
**  Compute the spectrum as a function of the system matrices
**
**  spec_mat    = spec_dsge(TT,RR,ZZ,QQ,om_grid);
**
**  Last updated: 10/21/2005 
********************************************************************/
proc (1)    = dsge_spectrum(TT,RR,ZZ,QQ,om_grid);    /** HH=0 & VV=0 **/

/** 
global intidx_, cumidx_
**/
local   om_n,nstate,ny,spec_mat,im_i,
        Sl,SD,Ddiag,Dinv,om_idx,om;
@       nr,p,d,q,TTsvd,RRsvd,Ptsvd,Pt,GAM;@

/** dimensions  **/
om_n     =  rows(om_grid);
nstate   =  rows(TT);
ny       =  rows(ZZ);

/** initialize  output **/
spec_mat    = zeros(om_n,ny);

/** imaginary unit: i **/
let im_i    = 1i;

/** spectrum density    of  y **/
/** loop    along   omega_grid_ **/

om_idx =    1;
do  until   om_idx >    om_n;

    om  = om_grid[om_idx];

    Sl  = eye(nstate)-TT*exp(-im_i*om);
    SD  = ZZ*inv(Sl)*(RR*QQ*RR')*inv(Sl')*ZZ'/2/pi;
        
    spec_mat[om_idx,.]    = diag(SD)';

    om_idx =    om_idx+1;
endo;

retp(spec_mat);
endp;
