/******************************************************************************
* DSGE_SS.G
* for every draw of DSGE parameters, computes steady state values of variables 
*
* created:  03-SEP-2007
* based on: dsge_ac.g
******************************************************************************/
new;
closeall;
library pgraph, user;
dlibrary -a PszTgSen,PsdTgSen;

cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

#include dsge_spec.g;
#include pathspec.g;
#include dsge_mod.src;
#include evalmod.g;

msel = 2 | 1;

/********************************************************************
* Specify parameter values
********************************************************************/

goto m1m21t2p3para;

m1m21t2p2para:

del   = 0.0140;  /* depreciation rate */
alp   = 0.3221;  /* capital share     */
lam   = 0.1383;  /* markup - (1+lam)/lam is elasticity of production of final */
                 /*          good Y_t wrt intermeadite good Y(i)_t            */
gam   = 1.0000;  /* risk aversion parameter in HH utility B*X_t^(1-gam)/(1-gam) */
zet   = 0.8743;  /* (1-zet) prob of reoptimizing prices by firms */
gst   = 1.2000;  /* (1-1/gst) is govt spending share of GDP in ss */
FF    = 0;       /* fixed costs in production of intermediate goods */

chist = 1.0000;  /* steady state value of preference shock CHI(t)   */
iota  = 0.8906;  /* fraction of firms that use dynamic indexation */
psi1  = 2.0000;  /* response of int rate to inflation in MP rule */
psi2  = 1.5178;  /* response of int rate to real GDP in MP rule */

rho_gov = 0.8120;  /* persistence: govt expenditures process */
rho_chi = 0.9800;  /* persistence: preference shock */
rho_r   = 0.6918;  /* persistence of int rate in MP rule */
rho_z   = 0.9800;  /* persistence: technology shock */

eta     = 1.0000;    /* preference parameter in DM HH utility f-n */
kap     = 0.0001;    /* fixed - special lower bound in u(q) - DM utility    */
th      = 0.9535;    /* bargaining power of buyer */ 
sigmatilde = 0.6997; /* 2*probability to be a buyer/seller in DM */ 
psitilde= 0.0000;    /* estimated - reparameterization of psi */
                     /* psi - parameter in c(q) - seller's cost function in DM */
                     /* 0 means same as in CM */

pi_steady    = 4.0000;  /* steady state inflation */
rr_steady    = 2.5000;  /* steady state real rate */
pi_steady_st = 0;       /* static indexation of firms that don't reoptimize prices */

Spp          =  3.6517 ;
lnmyp        =  -0.3800 ;
lnhstyst     =  -3.3500 ; 
lnycalst     =  0.0000  ;

PItarget0    = 0.0443;

sigGOV = 0.9879;  /* Stdev: shock to govt expenditures process */
sigCHI = 1.7990;  /* Stdev: preference shock */
sigR   = 0.3967;  /* Stdev: monetary policy shock */
sigZ   = 1.1120;  /* Stdev: technology shock */

sigPI  = 0.0486; 
sigR2  = 0.8926;

goto endpara;

m1m21t2p3para:

del   = 0.0140;  /* depreciation rate */
alp   = 0.3236;  /* capital share     */
lam   = 0.1367;  /* markup - (1+lam)/lam is elasticity of production of final */
                 /*          good Y_t wrt intermeadite good Y(i)_t            */
gam   = 1.0000;  /* risk aversion parameter in HH utility B*X_t^(1-gam)/(1-gam) */
zet   = 0.8301;  /* (1-zet) prob of reoptimizing prices by firms */
gst   = 1.2000;  /* (1-1/gst) is govt spending share of GDP in ss */
FF    = 0;       /* fixed costs in production of intermediate goods */

chist = 1.0000;  /* steady state value of preference shock CHI(t)   */
iota  = 0.7235;  /* fraction of firms that use dynamic indexation */
psi1  = 1.7000;  /* response of int rate to inflation in MP rule */
psi2  = 0.8620;  /* response of int rate to real GDP in MP rule */

rho_gov = 0.8394;  /* persistence: govt expenditures process */
rho_chi = 0.9749;  /* persistence: preference shock */
rho_r   = 0.6069;  /* persistence of int rate in MP rule */
rho_z   = 0.8278;  /* persistence: technology shock */

eta     = 1.0000;    /* preference parameter in DM HH utility f-n */
kap     = 0.0001;    /* fixed - special lower bound in u(q) - DM utility    */
th      = 0.9530;    /* bargaining power of buyer */ 
sigmatilde = 0.6298; /* 2*probability to be a buyer/seller in DM */ 
psitilde= 0.0000;    /* estimated - reparameterization of psi */
                     /* psi - parameter in c(q) - seller's cost function in DM */
                     /* 0 means same as in CM */

pi_steady    = 4.0000;  /* steady state inflation */
rr_steady    = 2.5000;  /* steady state real rate */
pi_steady_st = 0;       /* static indexation of firms that don't reoptimize prices */

Spp          =  4.8891 ;
lnmyp        =  -0.3800;
lnhstyst     =  -3.3500 ; 
lnycalst     =  0.0000  ;

PItarget0    = 0.0535;

sigGOV = 1.0090;  /* Stdev: shock to govt expenditures process */
sigCHI = 1.8027;  /* Stdev: preference shock */
sigR   = 0.3624;  /* Stdev: monetary policy shock */
sigZ   = 1.0354;  /* Stdev: technology shock */

sigPI  = 0.0486; 
sigR2  = 0.8477;

goto endpara;

m1m21t2p5para: 

del   = 0.0140;  /* depreciation rate */
alp   = 0.2881;  /* capital share     */
lam   = 0.1643;  /* markup - (1+lam)/lam is elasticity of production of final */
                 /*          good Y_t wrt intermeadite good Y(i)_t            */
gam   = 1.0000;  /* risk aversion parameter in HH utility B*X_t^(1-gam)/(1-gam) */
zet   = 0.7875;  /* (1-zet) prob of reoptimizing prices by firms */
gst   = 1.2000;  /* (1-1/gst) is govt spending share of GDP in ss */
FF    = 0;       /* fixed costs in production of intermediate goods */

chist = 1.0000;  /* steady state value of preference shock CHI(t)   */
iota  = 0.1411;  /* fraction of firms that use dynamic indexation */
psi1  = 1.7000;  /* response of int rate to inflation in MP rule */
psi2  = 0.8685;  /* response of int rate to real GDP in MP rule */

rho_gov = 0.8675;  /* persistence: govt expenditures process */
rho_chi = 0.9148;  /* persistence: preference shock */
rho_r   = 0.6525;  /* persistence of int rate in MP rule */
rho_z   = 0.7742;  /* persistence: technology shock */

eta     = 1.0000;    /* preference parameter in DM HH utility f-n */
kap     = 0.0001;    /* fixed - special lower bound in u(q) - DM utility    */
th      = 0.9615;    /* bargaining power of buyer */ 
sigmatilde = 0.1296; /* 2*probability to be a buyer/seller in DM */ 
psitilde= 0.0000;    /* estimated - reparameterization of psi */
                     /* psi - parameter in c(q) - seller's cost function in DM */
                     /* 0 means same as in CM */

pi_steady    = 4.0000;  /* steady state inflation */
rr_steady    = 2.5000;  /* steady state real rate */
pi_steady_st = 0;       /* static indexation of firms that don't reoptimize prices */

Spp          =  5.3979 ;
lnmyp        =  -0.3800  ;
lnhstyst     =  -3.3500 ; 
lnycalst     =  0.0000  ;

PItarget0    = -0.6848;

sigGOV = 1.0869;  /* Stdev: shock to govt expenditures process */
sigCHI = 4.0787;  /* Stdev: preference shock */
sigR   = 0.3282;  /* Stdev: monetary policy shock */
sigZ   = 1.8938;  /* Stdev: technology shock */

sigPI  = 0.0487; 
sigR2  = 0.7776;

goto endpara;

endpara:

/* Decision rules at zero inflation */
/* pi_steady_st = 0; */

para =   
      del|alp|lam|gam|          zet|gst|FF|chist|iota|psi1|psi2| 
      rho_gov|rho_chi| rho_r| rho_z|      eta| kap| th| sigmatilde| psitilde|
      pi_steady| rr_steady| pi_steady_st| Spp| lnmyp| lnhstyst| lnycalst|
      PItarget0 |
      sigGOV| sigCHI| sigR| sigZ | sigPI | sigR2;


/********************************************************************
* Compute steady states
********************************************************************/

{sst, valid} = dsgess(para,msel);

/** Report Results **/
"";
"Steady State of   |    Value ";
outmat = sst_names(msel)' $~ ftocv(sst,5,5) ;
$outmat;

/********************************************************************
* Check for determinacy
********************************************************************/

{TT,QQ,RR,HH,DD,ZZ,VV,RC,QQ2}=sysmat(para,msel);

if (RC[1] == 1) and (RC[2]==1);
   "";
   "Determinacy";
else;
   "";
   "No Determinacy";
   "Return Code" RC';
   end;
endif;

goto computeirf;

/********************************************************************
* Compute autocovariances
********************************************************************/


{vd,ac} = dsgemom(para,msel);

let string draw_names = {
               "MU_GDPR", "MU_INFL", "MU_FFUNDS", "MU_M2R",
               
               "V_GDPR", "C0_GDPR_INFL", "C0_GDPR_FFUNDS", "C0_GDPR_M2R",
               "C0_INFL_GDPR", "V_INFL", "C0_INFL_FFUNDS", "C0_INFL_M2R",
               "C0_FFUNDS_GDPR", "C0_FFUNDS_INFL", "V_FFUNDS", "C0_FFUNDS_M2R",
               "C0_M2R_GDPR", "C0_M2R_INFL", "C0_M2R_FFUNDS", "V_M2R",
               
               "C1_GDPR", "C1_GDPR_INFL", "C1_GDPR_FFUNDS", "C1_GDPR_M2R",
               "C1_INFL_GDPR", "C1_INFL", "C1_INFL_FFUNDS", "C1_INFL_M2R",
               "C1_FFUNDS_GDPR", "C1_FFUNDS_INFL", "C1_FFUNDS", "C1_FFUNDS_M2R",
               "C1_M2R_GDPR", "C1_M2R_INFL", "C1_M2R_FFUNDS", "C1_M2R"
               };
             

draw_names = vec(draw_names)';

"";
"Autocovariance      |   Value";
outmat = draw_names' $~ ftocv(ac',4,4) ;
outmat;

let string draw_names = {
               "GDPR_GOV",   "GDPR_CHI",   "GDPR_R",   "GDPR_Z",
               "INFL_GOV",   "INFL_CHI",   "INFL_R",   "INFL_Z",
               "FFUNDS_GOV", "FFUNDS_CHI", "FFUNDS_R", "FFUNDS_Z",
               "M2R_GOV",    "M2R_CHI",    "M2R_R",    "M2R_Z"
             };
draw_names = vec(draw_names)';

"";
"Variance Decomposition |   Value";
outmat = draw_names' $~ ftocv(vec(vd'),4,4) ;
outmat;

/********************************************************************
* Impulse Responses
********************************************************************/
computeirf:

nirf   = 20;
ti     = seqa(1,1,nirf);
impind = 4;
impact = zeros(nvar,1);
impact[impind] = para[28+impind];

yyirf = dsgeirf(para,msel,nirf,impact); 

/* Plot Results */

graphset;
begwind;
margin(0,0,0.2,0.2);
window(2,2,0);
fonts("microb");
_ptitlht = 0.4;
_paxht = 0.3;
_pnumht = 0.4;

_protate = 0; 
_plctrl = 0;
_plwidth = 7.5;
_psymsiz = 5;
_protate = 0;
xtics(0,20,5,5);
_pline = ( 1 ~ 6 ~ 0 ~ 0 ~ maxc(ti) ~ 0 ~ 4 ~ 2);

_pltype = {6 1 3 };
_pcolor = {1 6 4 };

ytics(0,0.6,0.2,2); 
title("Output");
_plegstr = "GDP\000CM\000DM"; 
_plegctl = {2 4 2 1.2};
xy(ti, yyirf[.,1:3]);
nextwind;

ytics(-4,2,2,2); 
title("Inflation");
_plegstr = "GDP DEFL\000CM DEFL\000DM DEFL"; 
_plegctl = {2 4 2 1.2};
xy(ti, yyirf[.,4:6]);
nextwind;

ytics(0,1.5,0.5,2); 
title("Consumption and Investment");
_plegstr = "X(t)\000I(t)\000Q(t)"; 
_plegctl = {2 4 3.2 2};
xy(ti, yyirf[.,7:9]);
nextwind;

ytics(-1.5,0.5,0.5,2); 
title("Hours");
_plegctl = { };
xy(ti, yyirf[.,10]);
endwind;

end;

/********************************************************************
* Simulate Inflation Data
********************************************************************/
siminfldata:

nburn = 100;
ntotal = 200;
nsim  = ntotal - nburn;

yT = dsgeinflsim(para,msel,nburn,ntotal);

"Standard deviations";
stdc(yT)';

corr_pigdp = corrx(yT[2:nsim,1]~yT[1:nsim-1,1]);
corr_pigdp = corr_pigdp[1,2];

corr_picm = corrx(yT[2:nsim,2]~yT[1:nsim-1,2]);
corr_picm = corr_picm[1,2];

corr_pidm = corrx(yT[2:nsim,3]~yT[1:nsim-1,3]);
corr_pidm = corr_pidm[1,2];

"Autocorrelation";
corr_pigdp corr_picm corr_pidm;

"Simulated Inflation";
yT;

end;

/********************************************************************
* Simulate Data
********************************************************************/
simdata:

#include loaddata.g;
nobs  = rows(YY);          /** number of observations **/
ny    = cols(YY);          /** number of series **/

{lnpY,retcode,obserror,obsvar} = evalmod(para,YY,msel);
lnpY;

{yTG,yTchi,yTR,yTZ,yTpi,yTall} = dsgesim(para,msel,100,200);

graphset;

fonts("microb");
_ptitlht= 0.3;
_paxht  = 0.25;
_pnumht = 0.2;
_protate= 0;
_plctrl = -1;
_pltype = {6 2 3};

begwind;
margin(0.2,0.2,0.2,0.2);
window(3,2,0);  


/* Name the AXIS labels
*/
ylabel("log (Inverse Velocity)");
xlabel("Nominal Interest Rate (%)");
xtics(0, 15, 5,1);
ytics(-1,0,0.2,1);
/*
_pmsgstr = "D";
_pmsgctl =  (Tsamp[1] ~ Tsamp[2] ~ 0.35 ~ 0 ~ 1 ~ 4 ~ 1);
*/

title("Monetary Policy (Short-Run)");
_pcolor = 2;
_psymsiz = 1;
xy(yTR[.,3],yTR[.,4]/100);
_psymsiz = 3;
_pcolor = 4;
xy(yy[.,3],yy[.,4]/100);
nextwind;

title("Monetary Policy (Long-Run)");
_pcolor = 2;
_psymsiz = 1;
xy(yTpi[.,3],yTpi[.,4]/100);
_psymsiz = 3;
_pcolor = 4;
xy(yy[.,3],yy[.,4]/100);
nextwind;

title("Technology");
_pcolor = 2;
_psymsiz = 1;
xy(yTz[.,3],yTz[.,4]/100);
_psymsiz = 3;
_pcolor = 4;
xy(yy[.,3],yy[.,4]/100);
nextwind;

title("Government Spending");
_pcolor = 2;
_psymsiz = 1;
xy(yTg[.,3],yTg[.,4]/100);
_psymsiz = 3;
_pcolor = 4;
xy(yy[.,3],yy[.,4]/100);
nextwind;

title("Money Demand");
_pcolor = 2;
_psymsiz = 1;
xy(yTchi[.,3],yTchi[.,4]/100);
_psymsiz = 3;
_pcolor = 4;
xy(yy[.,3],yy[.,4]/100);
nextwind;

title("All Shocks");
_pcolor = 2;
_psymsiz = 1;
xy(yTall[.,3],yTall[.,4]/100);
_psymsiz = 3;
_pcolor = 4;
xy(yy[.,3],yy[.,4]/100);
endwind;


/* Compute some variances and correlations
*/

"Data / Model";
"Interest" stdc(YY[.,3])~stdc(yTpi[.,3]);
"Velocity" stdc(YY[.,4]/100)~stdc(yTpi[.,4]/100);
corrdata = corrx(YY[.,3]~YY[.,4]/100);
corrdata = corrdata[1,2];
corrmod = corrx(yTpi[.,3]~yTpi[.,4]/100);
corrmod = corrmod[1,2];
"Correlation" corrdata~corrmod;

end;

/* Estimate MD on simulated data
*/
nograph:

nysim = rows(yTall);

gdp      = yTall[3:nysim,1];
gdplag1  = yTall[2:nysim-1,1];
gdplag2  = yTall[1:nysim-2,1];

infl      = yTall[3:nysim,2];
infllag1  = yTall[2:nysim-1,2];
infllag2  = yTall[1:nysim-2,2];

ffunds      = yTall[3:nysim,3];
ffundslag1  = yTall[2:nysim-1,3];
ffundslag2  = yTall[1:nysim-2,3];
 
myp      = yTall[3:nysim,4];
myplag1  = yTall[2:nysim-1,4];
myplag2  = yTall[1:nysim-2,4];

mp      = myp + gdp;
mplag1  = myplag1 + gdplag1;
mplag2  = myplag2 + gdplag2;

intcpt  = ones(nysim-2,1);

"Model Data - Levels";
xx      = intcpt ~ gdp ~ ffunds ~ mplag1 ~ infl;
bethat = inv(xx'*xx)*xx'*mp;
coeff_names =    "incept"
              $| "gdp   " 
              $| "ffunds"
              $| "mp(1) "
              $| "infl  ";
outmat = coeff_names $~ ftocv(bethat,4,4) ;
$outmat;
"";
"";

"Model Data - Differences";
xx      = intcpt ~ (gdp-gdplag1) ~ (ffunds-ffundslag1) ~ (infl-infllag1);
bethat  = inv(xx'*xx)*xx'*(mp-mplag1);
coeff_names =    "incept   "
              $| "D(gdp)   " 
              $| "D(ffunds)"
              $| "D(infl)  ";
outmat = coeff_names $~ ftocv(bethat,4,4) ;
$outmat;
"";
"";

"Model Data - VECM";
xx      = intcpt ~ gdplag1 ~ ffundslag1 ~ mplag1 ~ (gdplag1-gdplag2) 
          ~ (ffundslag1-ffundslag2) ~ (mplag1-mplag2);
bethat = inv(xx'*xx)*xx'*(mp-mplag1);
coeff_names =    "incept    "
              $| "gdp(1)    "
              $| "ffunds(1) "
              $| "mp(1)     "
              $| "D1(gdp)   " 
              $| "D1(ffunds)"
              $| "D1(mp)    ";
outmat = coeff_names $~ ftocv(bethat,4,4) ;
$outmat;
"";
"";

"Model Data - Stock Watson";
xx      = intcpt ~ gdp ~ ffunds ~ (gdplag1-gdplag2) 
          ~ (ffundslag1-ffundslag2);
bethat = inv(xx'*xx)*xx'*mp;
coeff_names =    "incept    "
              $| "gdp       "
              $| "ffunds    "
              $| "D1(gdp)   " 
              $| "D1(ffunds)"
              ;
outmat = coeff_names $~ ftocv(bethat,4,4) ;
$outmat;
"";
"";

"Model Data - IV";
xx      = intcpt ~ gdp ~ ffunds;
zz      = intcpt ~ gdplag1 ~ ffundslag1;
bethat = inv(zz'*zz)*xx'*mp;
coeff_names =    "incept    "
              $| "gdp       "
              $| "ffunds    "
              ;
outmat = coeff_names $~ ftocv(bethat,4,4) ;
$outmat;
"";
"";


/* Estimate MD on actual data
*/

nysim = rows(yy);

gdp      = yy[3:nysim,1];
gdplag1  = yy[2:nysim-1,1];
gdplag2  = yy[1:nysim-2,1];

infl      = yy[3:nysim,2];
infllag1  = yy[2:nysim-1,2];
infllag2  = yy[1:nysim-2,2];

ffunds      = yy[3:nysim,3];
ffundslag1  = yy[2:nysim-1,3];
ffundslag2  = yy[1:nysim-2,3];

myp      = yy[3:nysim,4];
myplag1  = yy[2:nysim-1,4];
myplag2  = yy[1:nysim-2,4];

mp      = myp + gdp;
mplag1  = myplag1 + gdplag1;
mplag2  = myplag2 + gdplag2;

intcpt  = ones(nysim-2,1);

"Actual Data - Levels";
xx      = intcpt ~ gdp ~ ffunds ~ mplag1 ~ infl;
bethat = inv(xx'*xx)*xx'*mp;
coeff_names =    "incept"
              $| "gdp   " 
              $| "ffunds"
              $| "mp(1) "
              $| "infl  ";
outmat = coeff_names $~ ftocv(bethat,4,4) ;
$outmat;
"";
"";

"Actual Data - Differences";
xx      = intcpt ~ (gdp-gdplag1) ~ (ffunds-ffundslag1) ~ (infl-infllag1);
bethat  = inv(xx'*xx)*xx'*(mp-mplag1);
coeff_names =    "incept   "
              $| "D(gdp)   " 
              $| "D(ffunds)"
              $| "D(infl)  ";
outmat = coeff_names $~ ftocv(bethat,4,4) ;
$outmat;
"";
"";

"Actual Data - VECM";
xx      = intcpt ~ gdplag1 ~ ffundslag1 ~ mplag1 ~ (gdplag1-gdplag2) 
          ~ (ffundslag1-ffundslag2) ~ (mplag1-mplag2);
bethat = inv(xx'*xx)*xx'*(mp-mplag1);
coeff_names =    "incept    "
              $| "gdp(1)    "
              $| "ffunds(1) "
              $| "mp(1)     "
              $| "D1(gdp)   " 
              $| "D1(ffunds)"
              $| "D1(mp)    ";
outmat = coeff_names $~ ftocv(bethat,4,4) ;
$outmat;
"";
"";

"Actual Data - Stock Watson";
xx      = intcpt ~ gdp ~ ffunds ~ (gdplag1-gdplag2) 
          ~ (ffundslag1-ffundslag2);
bethat = inv(xx'*xx)*xx'*mp;
coeff_names =    "incept    "
              $| "gdp       "
              $| "ffunds    "
              $| "D1(gdp)   " 
              $| "D1(ffunds)"
              ;
outmat = coeff_names $~ ftocv(bethat,4,4) ;
$outmat;
"";
"";


end;


/********************************************************************
* Procedure: DSGEMOM
********************************************************************/
proc (2) = dsgemom(para,msel);

local TT,QQ,QQ2,RR,HH,DD,ZZ,VV,RC,sig_chol,i,ny,
      GAM0s,GAM1s,GAM0i,GAM1i,GAM0,GAM1,QQi,GAM0tr,seldiag,seloffdiag,
      vd,ac;

/**********************************************************
**      System matrices for State-space
**********************************************************/
{TT,QQ,RR,HH,DD,ZZ,VV,RC,QQ2}=sysmat(para,msel);
ny = rows(ZZ);

/**********************************************************
**   variance decomposition and autocovariance
**********************************************************/

/** sqrt decomposition **/
sig_chol = sqrt(QQ);

GAM0 = zeros(ny,ny);
GAM1 = zeros(ny,ny);
vd   = zeros(ny,cols(sig_chol));

i=1;
do until i > cols(sig_chol);

   QQi   = sig_chol[.,i]*sig_chol[.,i]';
   GAM0s = real(dlyap(TT,RR*QQi*RR'));
   GAM0i = ZZ*GAM0s*ZZ' + ZZ*RR*VV + (ZZ*RR*VV)' + HH;
   GAM1i = ZZ*(TT*GAM0s)*ZZ' + ZZ*(TT*RR*VV);

   GAM0 = GAM0 + GAM0i;
   GAM1 = GAM1 + GAM1i;

   vd[.,i] = diag(GAM0i);

   i = i+1;
endo;

/** variance decomposition **/
vd = vd./diag(GAM0);

/** autocovariance **/

GAM0tr     = sqrt(diag(GAM0))*( sqrt(diag(GAM0)) )';
seldiag    = eye(rows(GAM0));
seloffdiag = ones(rows(GAM0),cols(GAM0)) - seldiag;
      
GAM0   = seldiag.*sqrt(GAM0tr) + seloffdiag.*( GAM0./GAM0tr);
GAM1   = GAM1./GAM0tr;

/** ny + ny^2 + ny^2 **/
ac = DD' ~ vec(GAM0)' ~ vec(GAM1)';

retp(vd,ac);
endp;

/********************************************************************
* Procedure: DSGESIM
********************************************************************/

proc (6) = dsgesim(para,msel,Tsimburn,Tsimtotal);
local TT,QQ,QQ2,RR,HH,DD,ZZ,VV,RC,
      HHchol,QQchol,neps,nstate,ny,
      ytG, ytchi, ytR, ytZ, ytpi,
      stG, stchi, stR, stZ, stpi,
      epst,t;

/* Compute system matrices for state space representation
*/
{TT,QQ,RR,HH,DD,ZZ,VV,RC,QQ2}=sysmat(para,msel);

neps   = cols(QQ);
nstate = cols(TT);
ny     = rows(ZZ);

QQchol = cholpsd(QQ);

/* Simulate DSGE model
*/

/* Initializations 
*/
yTG  = zeros(Tsimtotal+Tsimburn,ny);
yTchi= zeros(Tsimtotal+Tsimburn,ny);
yTR  = zeros(Tsimtotal+Tsimburn,ny);
yTZ  = zeros(Tsimtotal+Tsimburn,ny);
yTpi = zeros(Tsimtotal+Tsimburn,ny);

stG   = zeros(nstate,1);
stchi = zeros(nstate,1);
stR   = zeros(nstate,1);
stZ   = zeros(nstate,1);
stpi  = zeros(nstate,1);

/* Iterations
*/
t = 1;
do until t > Tsimburn + Tsimtotal;

   /* calculate realization of epst */
   epst   = QQchol*rndn(neps,1);
  
   /* Update vector x(t) */
   stG   = TT*stG   + RR*( (1|0|0|0|0) .* epst);
   stchi = TT*stchi + RR*( (0|1|0|0|0) .* epst);
   stR   = TT*stR   + RR*( (0|0|1|0|0) .* epst);
   stZ   = TT*stZ   + RR*( (0|0|0|1|0) .* epst);
   stpi  = TT*stpi  + RR*( (0|0|0|0|1) .* epst);
   
   yTG[t,.]   = (ZZ*stG)';
   yTchi[t,.] = (ZZ*stchi)';
   yTR[t,.]   = (ZZ*stR)';
   yTZ[t,.]   = (ZZ*stZ)';
   yTpi[t,.]  = (ZZ*stpi)';

   t = t+1;

endo;

yTG   = yTG[Tsimburn+1:Tsimtotal+Tsimburn,.];
yTchi = yTchi[Tsimburn+1:Tsimtotal+Tsimburn,.];
yTR   = yTR[Tsimburn+1:Tsimtotal+Tsimburn,.];
yTZ   = yTZ[Tsimburn+1:Tsimtotal+Tsimburn,.];
yTpi  = yTpi[Tsimburn+1:Tsimtotal+Tsimburn,.];
yTall = DD'+yTG+yTR+yTchi+yTz+yTpi;


retp(yTG+DD',yTchi+DD',yTR+DD',yTZ+DD',yTpi+DD',yTall);
endp;


/********************************************************************
* Procedure: DSGEINFLSIM
********************************************************************/

proc (1) = dsgeinflsim(para,msel,Tsimburn,Tsimtotal);
local TT,QQ,QQ2,RR,HH,DD,ZZ,VV,RC,
      PI_t, PIdm_t, PIgdp_t,
      HHchol,QQchol,neps,nstate,ny,
      yT, sT, epst,t, MC_t;

/* Compute system matrices for state space representation
*/
{TT,QQ,RR,HH,DD,ZZ,VV,RC,QQ2}=sysmat(para,msel);


QQ[5,5] = 0;
neps   = cols(QQ);
nstate = cols(TT);
QQchol = cholpsd(QQ);

/* Redefine the measurement equation
*/

MC_t     = 8;
PI_t     = 9;
PIdm_t   = 54;
PIgdp_t  = 55;

ZZ = zeros(4,nstate);

/* actual inflation */
ZZ[1,PIgdp_t] = 4;

/* CM Inflation */
ZZ[2,PI_t] = 4;

/* DM Inflation */
ZZ[3,PIdm_t] = 4;

/* MC Inflation */
ZZ[4,MC_t] = 1;



ny     = rows(ZZ);

/* Initializations 
*/
yT   = zeros(Tsimtotal+Tsimburn,ny);
sT   = zeros(nstate,1);

/* Iterations
*/
t = 1;
do until t > Tsimburn + Tsimtotal;

   /* calculate realization of epst */
   epst   = QQchol*rndn(neps,1);
  
   /* Update vector x(t) */
   sT        = TT*sT   + RR*epst;
   yT[t,.]   = (ZZ*sT)';
   
   t = t+1;

endo;

yT = yT[Tsimburn+1:Tsimtotal+Tsimburn,.];

retp(yT);
endp;

/********************************************************************
**   Procedure to compute Impulse Response Function
********************************************************************/
proc (1) = dsgeirf(para,msel,nirf,imp);
local TT,QQ,QQ2,RR,HH,DD,ZZ,VV,RC, ssirf, yyirf, t,

      /*  - ordinary variables  */
      D_t,  F1_t, F2_t, H_t,  I_t,  K_tp1, M_tp1, MC_t, PI_t,
      PO_t, Q_t,  R_t,  Rk_t, UP_t, W_t,   X_t,   XI_t, Y_t, Ycal_t, Y_dot_t,
      BG_t, BGK_t, BGAM_t, BGAM_TEMP_t, BGQ_t, BU_t, BUP_t, BUPP_t, 
      BC_t, BCQ_t, BCK_t, BCQQ_t, BCKK_t, BCQK_t, mu_t,
      PItarget_t,
      /*  - jumpers */
      EPI_t, EPO_t, EF1_t, EXI_t, EF2_t, EUP_t, ERk_t, EBGAM_t, EBG_t, EBUP_t,
      EBGQ_t, EI_t, Emu_t,
      /*  - exogenous processes */
      GOV_t, CHI_t, MP_t, Z_t,
      /* additional variables to construct GDP */
      M_tp1lag, PIdm_t, PIgdp_t, Ygdp_t
      ;

/**********************************************************
**      System matrices for State-space
**********************************************************/

{TT,QQ,RR,HH,DD,ZZ,VV,RC,QQ2}=sysmat(para,msel);

/**********************************************************
**      Variable Indices for TT matrix
**********************************************************/

/** variable indices **/

D_t   = 1;    /* ordinary variables */
F1_t  = 2;
F2_t  = 3;
H_t   = 4;
I_t   = 5;
K_tp1 = 6;
M_tp1 = 7;
MC_t  = 8;
PI_t  = 9;
PO_t  = 10;
Q_t   = 11;
R_t   = 12;
Rk_t  = 13;
UP_t  = 14;
W_t   = 15;
X_t   = 16;
XI_t  = 17;
Y_t   = 18;
Ycal_t  = 19;
Y_dot_t = 20;
BG_t  = 21;
BGK_t = 22;
BGAM_t= 23;
BGAM_TEMP_t = 24;
BGQ_t = 25;
BU_t  = 26;
BUP_t = 27;
BUPP_t= 28; 
BC_t  = 29;
BCQ_t = 30;
BCK_t = 31;
BCQQ_t= 32;
BCKK_t= 33;
BCQK_t= 34;
EPI_t = 35;   /* period t expectations of t+1 terms */
EPO_t = 36;
EF1_t = 37;
EXI_t = 38;
EF2_t = 39;
EUP_t = 40;
ERk_t = 41;
EBGAM_t = 42;
EBG_t = 43;
EBUP_t= 44;
EBGQ_t= 45;
GOV_t = 46;   /* exogenous processes driven purely by shocks */
CHI_t = 47;
MP_t  = 48;
Z_t   = 49;
mu_t  = 50;
Emu_t = 51;
EI_t  = 52;
M_tp1lag = 53; /* additional variables to construct GDP */
PIdm_t   = 54;
PIgdp_t  = 55;
Ygdp_t   = 56;

PItarget_t = 58;

/**********************************************************
**      create a new measurement equations (ZZ)
**********************************************************/

ZZ = zeros(10,rows(TT));

/* GDP */
ZZ[1,Ygdp_t] =  1;

/* CM Output */
ZZ[2,Y_t]=1;

/* DM Output */
ZZ[3,M_tp1lag] = 1; ZZ[3,PI_t] = -1;

/* Agg inflation */
ZZ[4,PIgdp_t] = 4;

/* CM Inflation */
ZZ[5,pi_t] = 4;

/* DM Inflation */
ZZ[6,PIdm_t] = 4;

/* CM Consumption */
ZZ[7,X_t] = 1;

/* CM Investment */
ZZ[8,I_t] = 1;

/* DM Consumption */
ZZ[9,Q_t] = 1;

/* Hours worked */
ZZ[10,H_t] = 1; 

/* Compute IRFs */

ssirf = zeros(nirf,cols(ZZ));
yyirf = zeros(nirf,rows(ZZ));

ssirf[1,.] = (RR*imp)';
yyirf[1,.] = ssirf[1,.]*ZZ';
t = 2;
do until t > nirf;
   ssirf[t,.] = (TT*ssirf[t-1,.]')';
   yyirf[t,.] = ssirf[t,.]*ZZ';
   t = t+1;
endo;

eofirf:

retp(yyirf);
endp;
