new;
cls;
library user, pgraph;

/* Load Data
** GDP Deflator: 1947Q1 to 2007Q2 = 242 observations
*/
load GDPDEF[242,1] = "d:\\projects\\active\\moneysearch\\gauss\\Public\\data\\bandpassfilter\\gdpdef.txt";

/* Convert GDP Deflator into annualized quarter-to-quarter inflation rates
*/
Infl = 400*( ln(GDPDEF[2:242,1])-ln(GDPDEF[1:241,1]) );
nobs = rows(Infl);
Ti = seqa(1947.25,0.25,nobs); 

/* Filter obs DropBegin+1 to nobs-DropEnd 
** 1964Q1 - 2005Q1
*/
DropBegin = sumc(Ti .< 1964); 
DropEnd   = sumc(Ti .> 2005);
TiTrimmed   = Ti[DropBegin+1:nobs-DropEnd];
InflTrimmed = Infl[DropBegin+1:nobs-DropEnd];

/* Default Christiano-Fitzgerald filter 
*/
CFFilterDefault:  
  
   /* configure filter */
   pl = 2;
   pu = 64;
   
   /* apply filter */

   InflDevCF    = bpassCF(Infl,pl,pu);
   InflTrendCF  = Infl - InflDevCF; 
   
   /* trim beginning and end
   */
   
   InflDevCFTrimmed   = InflDevCF[DropBegin+1:nobs-DropEnd];
   InflTrendCFTrimmed = InflTrendCF[DropBegin+1:nobs-DropEnd];

/* Geweke Filter
*/

   /* configure filter
   */
   pl = 2;
   pu = 64;
   p  = 500;
   f  = 2;
   
   nLags  = 4;
   nTerms = 1000;
   
   BeginEstimation = DropBegin+1;
   EndEstimation   = nobs-DropEnd;
   
   InflDevGewekeTrimmed 
      = bpassGeweke(Infl,pl,pu,p,f,DropBegin,DropEnd,nLags,
                    BeginEstimation,EndEstimation,nTerms);
                    
   InflTrendGewekeTrimmed = InflTrimmed - InflDevGewekeTrimmed;
   
   
/* Plot results 
*/

graphset;
fonts("microb");
_ptitlht = 0.2;
_paxht   = 0.2;
_pnumht  = 0.2;
_pltype = {1 3 6 }; 
_pcolor = {4 1 2 };

_plegstr = "CF-Filter (Two-Sided)\000GP-Filter (One-Sided)\000Inflation"; 
_plegctl = {2 4 5 4}; 
_plwidth = 7;

begwind;
window(1,1,0);
xy( TiTrimmed, InflTrendCFTrimmed~InflTrendGewekeTrimmed~InflTrimmed );
endwind;

end;

/*****************************************************************************************
*/

proc bpassGeweke(Infl,pl,pu,p,f,DropBegin,DropEnd,nLags,
                 BeginEstimation,EndEstimation,nTerms);

local nobsFilter,i,InflDevTrimmed,
      a, b, n, BB0, BB,
      XX, YY, PhiHat,
      ObsIndex, nLagTerms, nLeadTerms, nLagTermsMissing, nLeadTermsMissing,
      InflVec, InflPred, j ;

nobsFilter = rows(Infl)-DropBegin-DropEnd;
InflDevTrimmed = zeros(nobsFilter,1);

/* Construct weights for ideal filter
*/

a   = 2*pi/pu;
b   = 2*pi/pl;
n   = seqa(1,1,nTerms);

BB0            = (b-a)/pi;
BB             = (sin(n*b) - sin(n*a)) ./ (pi*n);
BB             = BB0 | BB;
BB[nTerms+1,1] = -( BB[1,1]/2+sumc(BB[2:nTerms,1]) );
BB             = rev(BB[2:nTerms+1]) | BB;

/* Estimate VAR(p)
*/

YY = infl[BeginEstimation:EndEstimation];
XX = ones(rows(YY),1);
i = 1;
do until i > nLags;
   XX = XX~infl[BeginEstimation-i:EndEstimation-i];
   i = i+1;
endo;
PhiHat = inv(XX'*XX)*XX'YY; 

/* Apply the filter
*/

i = 1;
do until i > nobsFilter;

   ObsIndex   = DropBegin+i;
   
   nLagTerms  = ObsIndex-1;
   nLeadTerms = rows(Infl)-ObsIndex;

   nLagTerms  = minc(nTerms|nLagTerms|p);
   nLeadTerms = minc(nTerms|nLeadTerms|f);
   
   nLagTermsMissing  = maxc( 0 | (nTerms-nLagTerms)  );
   nLeadTermsMissing = maxc( 0 | (nTerms-nLeadTerms) );
   
   InflVec = infl[ObsIndex-nLagTerms:ObsIndex+nLeadTerms];
   
   if nLagTermsMissing > 0;
      /* backcasting */
      InflPred = 1 | InflVec[1:nLags];
      j = 1;
      do until j > nLagTermsMissing;
         InflVec  = (InflPred'*PhiHat) | InflVec;
         InflPred = 1 | InflVec[1:nLags];
         j = j+1;
      endo;
   endif;
   
   if nLeadTermsMissing > 0;
      /* forecasting */
      InflPred = 1 | InflVec[rows(InflVec)-nLags+1:rows(InflVec)];
      j = 1;
      do until j > nLeadTermsMissing;
         InflVec  = InflVec | (InflPred'*PhiHat);
         InflPred = 1 | InflVec[rows(InflVec)-nLags+1:rows(InflVec)]; 
         j = j+1;
      endo;      
   endif;
   i nLeadTerms nLeadTermsMissing nLagTerms nLagTermsMissing rows(BB) rows(InflVec);
   InflDevTrimmed[i,1] = BB'*InflVec;

   i = i+1;
endo;
                    
retp(InflDevTrimmed);
endp;



/*****************************************************************************************
*/

proc bpassCF(X,pl,pu);                					
/*			      GAUSS COMMAND FOR BAND PASS FILTER: bpass(X,pl,pu)

		This is a Gauss program that filters time series data using an approximation to the 
		band pass filter as discussed in the paper "The Band Pass Filter" by Lawrence J. 
		Christiano and Terry J. Fitzgerald (1999).									

		Required Inputs:
		X     - series of data (T x 1)
		pl    - minimum period of oscillation of desired component 
		pu    - maximum period of oscillation of desired component (2<=pl<pu<infinity).

		Output:
		fX - matrix (T x 1) containing filtered data 

		Examples: 
 
		Quarterly data: pl=6, pu=32 returns component with periods between 1.5 and 8 yrs.
		Monthly data: pl=2, pu=24 returns component with all periods less than 2 yrs.

		Note:  When feasible, we recommend dropping 2 years of data from the beginning 
		       and end of the filtered data series.  These data are relatively poorly
		       estimated.

		=====================================================================
		This program contains only the default filter recommended in Christiano and 
		Fitzgerald (1999). This program was written by Eduard Pelz and any errors are 
		my own and not the authors. For those who wish to use optimal band-pass filters
		other than the default filter please use the Matlab version of code available 
		at www.clev.frb.org/economists/fitzgerald.
		=====================================================================

		Version Date: 2/09/00 (Please notify Eduard Pelz at eduard.pelz@clev.frb.org 
		or (216)579-2063 if you encounter bugs).  
*/
local T,Xun,undrift ,i,drift,a,b,bnot,bhat0,n,bigB,AA,bhatu,bhatd,fX;		/*   local variable declaration */
/*
print "The lower and upper bounds are:";
format /rdn 1,0;
print "pl = " pl;
print "pu = " pu;
*/

format /ld  6,6;

if pl >= pu;
print "Lower bound (pl) must be less than upper bound (pu).";
end;
endif;
if pl < 2;
print "Lower bound (pl) must be greater than 2.";
end;
endif;

T = maxc(rows(X));
Xun = X;

/*This section removes the drift from a time series using the 
     formula:  drift = (X(T) - X(1)) / (T-1).                     
*/
undrift = 0;
if undrift == 1;
	drift = (X[T,1]-X[1,1])/(T-1);
	i = 1;
	do while i <= T;
	Xun[i,1] = X[i,1] - (i-1)*drift;
	i=i+1;
	endo;
endif;


/*Create the ideal B's then construct the AA matrix*/

a = 2*pi/pu;
b = 2*pi/pl;
bnot = (b-a)/pi;
bhat0 = bnot/2;

n = seqa(1,1,T);
bigB = (sin(n*b) - sin(n*a)) ./ (pi*n);
bigB[2:T,1] = bigB[1:T-1,1];
bigB[1,1] = bnot;

/*
"bigB";
bigB';
*/

AA = zeros(2*T,2*T);
i = 1;
do while i <= T;
	AA[i:i+T-1,i] = bigB;
	AA[i,i:i+T-1] = bigB';	
i = i+1;
endo;
AA = AA[1:T,1:T];

bhatu = zeros(T,1);
bhatd = zeros(T,1);
bhatu[1,1] = bhat0;
bhatd[T,1] = bhat0;

i = 1;
do while i <= T-1;
	bhatu[i+1,1] = bhatu[i,1]-bigB[i,1];
	bhatd[T-i,1] = bhatu[i,1]-bigB[i,1];
	i = i+1;
endo;

AA[1:T,1] = bhatu;
AA[1:T,T] = bhatd;

/*Filter data using AA matrix*/

fX = AA * Xun;

/*
"Results - Default";
Xun';
AA[11,.];
AA[11,.]*Xun;
sumc(AA[11,.]');
*/

retp(fX);
endp;



