'set work directory
cd c:\projects\active\moneysearch\data\

'create database
db msearch_us

'====================================

'create workfile for monthly data
wfcreate msearch_us_m M 1959:01 2006:12

  read(t=xls, s=Monthly, b134) moneysearch.xls CNP16OV FEDFUNDS M1SL M2SL

  read(t=xls, s=M2SNS, b13) m2sns.xls M2SNS
M2SNS.x12(mode=m) M2SNS

  read(t=xls, s=M2SNS, b13) m1sns.xls M1SNS
M1SNS.x12(mode=m) M1SNS

  store CNP16OV FEDFUNDS M2SL M2SNS_SA M1SL M1SNS_SA

save msearch_us_m
close msearch_us_m

'====================================

'create workfile for annual data
  wfcreate msearch_us_a A 1959 2006

  read(t=xls, c37) NIPA-FAT11.xls FACD
  read(t=xls, c37) NIPA-FAT13.xls FACDDEP

  store FACD FACDDEP

save msearch_us_a
close msearch_us_a

'====================================

'create workfile for quarterly data
wfcreate msearch_us_q Q 1959:01 2006:04

  read(t=xls, s=Quarterly, b50) moneysearch.xls GDP GDPC96 GDPCTPI GDPDEF GDPI GDPIC96 PCDG PCDGCC96 PCESV PCESVC96 PCND PCNDGC96

  read(t=xls,c64) NIPA-TABLE21.xls COMP

  read(t=xls,b43) DSSW-hours.xls HOURS
  
  'extract monthly series from database, convert to quarterly by averaging
  fetch(c=a) CNP16OV FEDFUNDS

  'extract monthly series from database, use value from last month of quarter
  fetch(c=l) M2SL M2SNS_SA M1SL M1SNS_SA

  'extract annual series from database, convert to quarterly using constants
  fetch(c=r) FACD 

  'extract annual series from database, convert to quarterly using constants
  fetch(c=d) FACDDEP

  'load target inflation series
  smpl 1964:01 2005:01
  read(t=txt) inflationtarget1964q12005q1.txt DSGE_INFLTARGET
  smpl @first @last

'====================================

'Compute GDP per capita
  genr gdpr = log(1E6*GDPC96/(4*CNP16OV))

'Compute inverse velocity as nominal ratios
  genr m2ry_n        = log(M2SL/(GDP/4))
  genr m2sry_n       = log(M2SNS_SA/(GDP/4))
  genr m1ry_n        = log(M1SL/(GDP/4))
  genr m1sry_n       = log(M1SNS_SA/(GDP/4))

'Compute depreciation rates
  genr delta = FACDDEP/FACD

'Compute labor share
  genr laborshare = COMP/GDP

'Compute consumption share
  genr consumshare_n = (PCND+PCESV+PCDG)/GDP
  genr consumshare_r = (PCNDGC96+PCESVC96+PCDGCC96)/GDPC96

'Compute investment share
 genr investshare_n = (GDPI)/GDP
 genr investshare_r = (GDPIC96)/GDPC96

'Compute gstar
  genr gstar_n = GDP/(GDPI+PCDG+PCND+PCESV)
  genr gstar_r = GDPC96/(GDPIC96+PCDGCC96+PCNDGC96+PCESVC96)

'Compute annualized inflation rates
  genr infl_annual = 400*(log(GDPDEF)-log(GDPDEF(-1)))

'Compute annualized real rates (ex post)
  genr realrate_annual = fedfunds(-1)-infl_annual

'====================================

'Detrending

'Inv M2 velocity, 0.814 is 1965q1 - 2005q1 mean
  m2sry_n.hpf(lambda=1600) m2sry_n_hptr
  series m2sry_n_hpdev = 0.814 + m2sry_n - m2sry_n_hptr

  equation eq_m2sry.ls m2sry_n c @trend
  series m2sry_n_olstr = m2sry_n - resid
  series m2sry_n_olsdev = 0.814+ resid

'Inv M1 velocity, -0.384 is 1965q1 - 2005q1 mean

  m1sry_n.hpf(lambda=1600) m1sry_n_hptr
  series m1sry_n_hpdev = -0.384 + m1sry_n - m1sry_n_hptr

  series dumpre1982 = @year < 1982
  series dumpost1982 = 1-dumpre1982

  equation eq_m1sry.ls m1sry_n dumpre1982 dumpost1982 dumpre1982*@trend dumpost1982*@trend

  series m1sry_n_olstr = m1sry_n - resid
  series m1sry_n_olsdev = -0.384 + resid

'GDP real
  gdpr.hpf(lambda=1600) gdpr_hptr
  series gdpr_hpdev = gdpr - gdpr_hptr

  equation eq_gdpr.ls gdpr c @trend
  series gdpr_olstr   = gdpr - resid
  series gdpr_olsdev = resid

'change sample to estimation period 1964q1 to 2005q1
'scale the observables 
  
  smpl 1964q1 2005q1

  series dsge_output_lt = 100*gdpr_olsdev
  series dsge_output_hp = 100*gdpr_hpdev

  series dsge_infl = infl_annual
  series dsge_infldev = dsge_infl - dsge_infltarget
  series dsge_interest = fedfunds
  series dsge_money_m1 = 100*m1sry_n  
  series dsge_money_m2 = 100*m2sry_n  
  
' Estimate AR models for target inflation rate
  smpl 1965q1 2005q1
  equation eq_targetar1.ls dsge_infltarget c AR(1)
  equation eq_targetar2.ls dsge_infltarget c AR(1) AR(2)
  equation eq_targetar4.ls dsge_infltarget c AR(1) AR(2) AR(3) AR(4)
  smpl 1964q1 2005q1

' Group Series: Linear Trend
 
  group dsge_ltm1 dsge_output_lt dsge_infl dsge_interest dsge_money_m1 dsge_infltarget

  group dsge_ltm2 dsge_output_lt dsge_infl dsge_interest dsge_money_m2 dsge_infltarget

' Group Series: HP Trend
  
  group dsge_hpm1 dsge_output_hp dsge_infl dsge_interest dsge_money_m1 dsge_infltarget

  group dsge_hpm2 dsge_output_hp dsge_infl dsge_interest dsge_money_m2 dsge_infltarget
  
save msearch_us_q
close msearch_us_q
close msearch_us

