/*************************************************************
** filename:    parasubsamp.g
** description: extracts a subsample from the posterior draws 
**              and stores it in an ASCII file
*************************************************************/

new;
closeall;
library user, pgraph;
cls;

format /mat /on /mb1 /ros 10,4;
outwidth 256;

mhrun  = "1";
nburn  = 10001;        
/* Number of initial draws to be discarded 
** use nburn = 1 for draws from prior
** use the equivalent of 20 percent for draws from posterior */
nblock = 1E5;
nuse   = 100;

#include dsge_mod.src;
#include dsge_para.src;
#include dsge_spec.g;

#include pathspec.g;

para  = pmsv(msel);
npara = rows(para);
{para,para_names} = psel(para,msel);

/******************************************************************
**         Initialize Input and Output files
******************************************************************/

lpath  = parapath $+ "\\mhrun" $+ mhrun;
opath  = parapath $+ "\\mhrun" $+ mhrun;

ldraw  = lpath $+ "\\pa"  ;
odraws = opath $+ "\\pasub" ;

open   fhdraw = ^ldraw for read;
create fhdrawsub = ^odraws with PARA, npara, 8;

/** start log file **/
outstr = opath $+ "\\pasub.log";
output file = ^outstr reset;
$para_names;

drawrow = seekr( fhdraw, nburn);

eofloop = 0;
ndraws  = 0;

do until eofloop; 

    /** read blocks of size nblock **/
    parasim = readr(fhdraw,nblock);
    nread   = rows(parasim);

    /** use every nuse-th draw **/
    indseq  = seqa(1,1,nread);
    indseq  = indseq % nuse;
    indseq  = indseq .ne 0;

    parasim = delif(parasim,indseq);
    nread   = rows(parasim);

    if ismiss(parasim) == 0;

       wr = writer(fhdrawsub,parasim);
       parasim;
       ndraws = ndraws+nread;

    endif;

    eofloop = eof(fhdraw);

endo;

closeall fhdraw, fhdrawsub;

/** close log file **/
output file = ^outstr off;
