/********************************************************************
*  Model Specification and Global Setting
*
*  Last updated: 05/31/2005
********************************************************************/
/********************************************************************
* "Estimation / Model Selection in Monetary Models"
* by B Aruoba, F. Schorfheide
*
*  Search-Based Monetary Model (Bargaining version)
*
* updated by Max Kryshko
* 06-Jul-2007, FRI
********************************************************************/
dlibrary -a PszTgSen,PsdTgSen;

msel = zeros(2,1);

datasel = 1;
msel[1] = 2;
msel[2] = 1;
subT    = 2;
mprior  = 31;

/************************************************
*  MSEL[1]:
*   1 benchmark model
*   2 PHI = 1/(1-alpha) (same technology in DM/CM)
************************************************/
lmodel = "m" $+ ftos(msel[1],"%*.*lf",1,0);

/************************************************
*  MSEL[2]:
*   0 no steady state penalty
*   1 with steady state penalty for markup and i/y
************************************************/
lmodel = lmodel $+ ftos(msel[2],"%*.*lf",1,0);

/************************************************
*  DATASEL
*   1  for M1 data, 
*   2  
************************************************/
if datasel==1;
   dataselstr = "m1_t2";
endif;

/************************************************
*  subT
*
*     0 for post 1984
*     1 for post 1987
*     2 for 1964q1 ...
*
************************************************/
subTstr = "t" $+ ftos(subT,"%*.*lf",1,0);

_tbegin2 = 0;
_tend2   = 0;

/************************************************
*  mprior
************************************************/
lprior = "p" $+ ftos(mprior,"%*.*lf",1,0);

/************************************************
*  ngrid: number of grids for density
************************************************/
ngrid = 100;

/************************************************
*  titlestr: titles used for various graphs
************************************************/
titlestr = "Real GDP per capita (cyc)" $|
           "Inflation (GDP deflator)" $|
           "Federal FUNDS Rate" $|
           "M1 inverse velocity" $|
           "Inflation Target" ;

/************************************************
* steady state targets: iy, markup, laborshare
*                       mean, std
************************************************/

if mprior == 1;
   _sstargets = ( 0.16 ~ 0.01 ) |
                ( 0.15 ~ 0.01 ) |
                ( 0.60 ~ 0.01 ) |
                ( 0.15 ~ 0.05 ) ;
elseif (mprior == 2) OR (mprior == 3) OR (mprior == 4) OR (mprior == 5) OR (mprior == 31);
   _sstargets = ( 0.16 ~ 0.005 ) |
                ( 0.15 ~ 0.01  ) |
                ( 0.60 ~ 0.01  ) |
                ( 0.15 ~ 0.05  ) ;
endif;

/************************************************
*  number of periods for impulse responses
************************************************/
nvar = 5;
nirf = 40;
