/******************************************************************************
* DSGE_SS.G
* for every draw of DSGE parameters, computes steady state values of variables 
*
* created:  03-SEP-2007
* based on: dsge_ac.g
******************************************************************************/
new;
closeall;
library pgraph, user;
dlibrary -a PszTgSen,PsdTgSen;

cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

#include dsge_spec.g;
#include pathspec.g;
#include dsge_mod.src;
msel = 2 | 1;

/********************************************************************
* Specify parameter values
********************************************************************/

goto m1m21t2p3para;

m1m21t2p3para:

del   = 0.0140;  /* depreciation rate */
alp   = 0.2735;  /* capital share     */
lam   = 0.1947;  /* markup - (1+lam)/lam is elasticity of production of final */
                 /*          good Y_t wrt intermeadite good Y(i)_t            */
gam   = 1.0000;  /* risk aversion parameter in HH utility B*X_t^(1-gam)/(1-gam) */
zet   = 0.8367;  /* (1-zet) prob of reoptimizing prices by firms */
gst   = 1.2000;  /* (1-1/gst) is govt spending share of GDP in ss */
FF    = 0;       /* fixed costs in production of intermediate goods */

chist = 1.0000;  /* steady state value of preference shock CHI(t)   */
iota  = 0.5682;  /* fraction of firms that use dynamic indexation */
psi1  = 1.7000;  /* response of int rate to inflation in MP rule */
psi2  = 0.8315;  /* response of int rate to real GDP in MP rule */

rho_gov = 0.8660;  /* persistence: govt expenditures process */
rho_chi = 0.9629;  /* persistence: preference shock */
rho_r   = 0.5967;  /* persistence of int rate in MP rule */
rho_z   = 0.8278;  /* persistence: technology shock */

eta     = 1.0000;    /* preference parameter in DM HH utility f-n */
kap     = 0.0001;    /* fixed - special lower bound in u(q) - DM utility    */
th      = 0.0000;    /* bargaining power of buyer - not used*/ 
sigmatilde = 0.5394; /* 2*probability to be a buyer/seller in DM */ 
psitilde= 0.0000;    /* estimated - reparameterization of psi */
                     /* psi - parameter in c(q) - seller's cost function in DM */
                     /* 0 means same as in CM */

pi_steady    = 4.0000;  /* steady state inflation */
rr_steady    = 2.5000;  /* steady state real rate */
pi_steady_st = 0;       /* static indexation of firms that don't reoptimize prices */

Spp          =  5.0839 ;
lnmyp        =  -0.3800  ;
lnhstyst     =  -3.3500 ; 
lnycalst     =  0.0000  ;

PItarget0    = 0.0234;

sigGOV = 1.0568;  /* Stdev: shock to govt expenditures process */
sigCHI = 1.8796;  /* Stdev: preference shock */
sigR   = 0.3670;  /* Stdev: monetary policy shock */
sigZ   = 1.0624;  /* Stdev: technology shock */

sigPI  = 0.0487; 
sigR2  = 0.8480;

goto endpara;

m1m21t2p5para: 

del   = 0.0140;  /* depreciation rate */
alp   = 0.2797;  /* capital share     */
lam   = 0.1713;  /* markup - (1+lam)/lam is elasticity of production of final */
                 /*          good Y_t wrt intermeadite good Y(i)_t            */
gam   = 1.0000;  /* risk aversion parameter in HH utility B*X_t^(1-gam)/(1-gam) */
zet   = 0.8034;  /* (1-zet) prob of reoptimizing prices by firms */
gst   = 1.2000;  /* (1-1/gst) is govt spending share of GDP in ss */
FF    = 0;       /* fixed costs in production of intermediate goods */

chist = 1.0000;  /* steady state value of preference shock CHI(t)   */
iota  = 0.1988;  /* fraction of firms that use dynamic indexation */
psi1  = 1.7000;  /* response of int rate to inflation in MP rule */
psi2  = 0.8804;  /* response of int rate to real GDP in MP rule */

rho_gov = 0.8696;  /* persistence: govt expenditures process */
rho_chi = 0.9116;  /* persistence: preference shock */
rho_r   = 0.6517;  /* persistence of int rate in MP rule */
rho_z   = 0.7517;  /* persistence: technology shock */

eta     = 1.0000;    /* preference parameter in DM HH utility f-n */
kap     = 0.0001;    /* fixed - special lower bound in u(q) - DM utility    */
th      = 0.0000;    /* bargaining power of buyer - not used*/ 
sigmatilde = 0.1266; /* 2*probability to be a buyer/seller in DM */ 
psitilde= 0.0000;    /* estimated - reparameterization of psi */
                     /* psi - parameter in c(q) - seller's cost function in DM */
                     /* 0 means same as in CM */

pi_steady    = 4.0000;  /* steady state inflation */
rr_steady    = 2.5000;  /* steady state real rate */
pi_steady_st = 0;       /* static indexation of firms that don't reoptimize prices */

Spp          =  5.4809 ;
lnmyp        =  -0.3800  ;
lnhstyst     =  -3.3500 ; 
lnycalst     =  0.0000  ;

PItarget0    = 0.0058;

sigGOV = 1.0861;  /* Stdev: shock to govt expenditures process */
sigCHI = 4.1127;  /* Stdev: preference shock */
sigR   = 0.3315;  /* Stdev: monetary policy shock */
sigZ   = 2.1283;  /* Stdev: technology shock */

sigPI  = 0.0487; 
sigR2  = 0.7981;

goto endpara;

endpara:

para =   
      del|alp|lam|gam|          zet|gst|FF|chist|iota|psi1|psi2| 
      rho_gov|rho_chi| rho_r| rho_z|      eta| kap| th| sigmatilde| psitilde|
      pi_steady| rr_steady| pi_steady_st| Spp| lnmyp| lnhstyst| lnycalst|
      PItarget0 |
      sigGOV| sigCHI| sigR| sigZ | sigPI | sigR2;


/********************************************************************
* Compute steady states
********************************************************************/

{sst, valid} = dsgess(para,msel);

/** Report Results **/
"";
"Steady State of   |    Value ";
outmat = sst_names(msel)' $~ ftocv(sst,5,5) ;
$outmat;

/********************************************************************
* Check for determinacy
********************************************************************/

{TT,QQ,RR,HH,DD,ZZ,VV,RC,QQ2}=sysmat(para,msel);

if (RC[1] == 1) and (RC[2]==1);
   "";
   "Determinacy";
else;
   "";
   "No Determinacy";
   "Return Code" RC';
   end;
endif;

goto dsgeirf;

/********************************************************************
* Compute and plot impulse responses
********************************************************************/

dsgeirf:
nirf     = 12;
nirfplot = nirf;
ti       = seqa(1,1,nirf);
impact   = 0 | 0 | para[31] | 0 | 0;
dsge_irf = dsgeirf(para,msel,nirf,impact);

titlestr = "CM Consumption (X)" $|
           "CM Inflation" $|
           "DM Consumption (Q)" $|
           "DM Inflation" $|
           "Total Consumption (in terms of CM goods)" $|
           "GDP Inflation";

drawtics = ( -1   ~ 0.5 ~ 0.5 )|
           ( -1   ~ 0.5 ~ 0.5   )|
           ( -1   ~ 0.5 ~ 0.5 )|
           ( -2   ~ 1   ~ 1 )|          
           ( -1   ~ 0.5 ~ 0.5 )|
           ( -1   ~ 0.5 ~ 0.5 );
              
graphset;
begwind;
margin(0,0,0.2,0.2);
window(3,2,0);
fonts("microb");
_ptitlht = 0.4;
_paxht = 0.3;
_pnumht = 0.35;
_pltype = {6 3 3 1};
_pcolor = {1 4 4 1};
_protate = 0; 
_plctrl = 0;
_plwidth = 6.5;
_psymsiz = 5;
_protate = 0;

pl_ind = 1;
do until pl_ind > 6;

   
   title(titlestr[pl_ind]);
   ytics(drawtics[pl_ind,1],drawtics[pl_ind,2],drawtics[pl_ind,3],2); 

   xy( ti[1:nirfplot],dsge_irf[1:nirfplot,pl_ind]);
         
   nextwind;
               
   pl_ind = pl_ind+1;
endo;
    
endwind;


/********************************************************************
* Procedure to compute impulse responses
********************************************************************/

/********************************************************************
**   Procedure to compute Impulse Response Function
********************************************************************/
proc (1) = dsgeirf(para,msel,nirf,imp);
local TT,QQ,QQ2,RR,HH,DD,ZZ,VV,RC, ssirf, yyirf, t,

      /*  - ordinary variables  */
      D_t,  F1_t,  F2_t,  H_t,  I_t,  K_tp1, M_tp1, MC_t, PI_t,
      PO_t, Q_t,   R_t,   Rk_t, UP_t, W_t,   X_t,   XI_t, Y_t, Ycal_t, Y_dot_t,
      BC_t, BCQ_t, BCK_t, mu_t,
      PItarget_t,
         
      /*  - jumpers */
      EPI_t, EPO_t, EF1_t, EXI_t, EF2_t, EUP_t, ERk_t, EBCK_t, EQ_t, 
      EBCQ_t, Emu_t, EI_t,

      /*  - exogenous processes */
      GOV_t, CHI_t, MP_t, Z_t,

      /* additional variables to construct GDP */
      M_tp1lag, PIdm_t, PIgdp_t, Ygdp_t
      ;

/**********************************************************
**      System matrices for State-space
**********************************************************/

{TT,QQ,RR,HH,DD,ZZ,VV,RC,QQ2}=sysmat(para,msel);

/**********************************************************
**      Variable Indices for TT matrix
**********************************************************/

/** variable indices **/

D_t   = 1;    /* ordinary variables */
F1_t  = 2;
F2_t  = 3;
H_t   = 4;
I_t   = 5;
K_tp1 = 6;
M_tp1 = 7;
MC_t  = 8;
PI_t  = 9;
PO_t  = 10;
Q_t   = 11;
R_t   = 12;
Rk_t  = 13;
UP_t  = 14;
W_t   = 15;
X_t   = 16;
XI_t  = 17;
Y_t   = 18;
Ycal_t  = 19;
Y_dot_t = 20;
BC_t  = 21;
BCQ_t = 22;
BCK_t = 23;
mu_t  = 24;
EPI_t = 25;   /* period t expectations of t+1 terms */
EPO_t = 26;
EF1_t = 27;
EXI_t = 28;
EF2_t = 29;
EUP_t = 30;
ERk_t = 31;
EBCK_t = 32;
EQ_t = 33;
EBCQ_t= 34;
Emu_t = 35;
EI_t  = 36;
GOV_t = 37;   /* exogenous processes driven purely by shocks */
CHI_t = 38;
MP_t  = 39;
Z_t   = 40;
M_tp1lag = 41; /* additional variables to construct GDP */
PIdm_t   = 42;
PIgdp_t  = 43;
Ygdp_t   = 44;

PItarget_t = 46;

/**********************************************************
**      create a new measurement equations (ZZ)
**********************************************************/

ZZ = zeros(6,rows(TT));

/* CM Consumption */
ZZ[1,X_t]=1;

/* CM Inflation */
ZZ[2,PI_t] = 4; 

/* DM Consumption (Units) */
ZZ[3,Q_t] = 1; 

/* DM Inflation */
ZZ[4,PIdm_t] = 4; 

/* Total Consumption in terms of CM Good */
ZZ[5,M_tp1lag] = 1*para[19/2]; ZZ[5,PI_t] = -1*para[19]/2; ZZ[5,X_t]=1;

/* Agg inflation */
ZZ[6,PIgdp_t] = 4;

/* Compute IRFs */

ssirf = zeros(nirf,cols(ZZ));
yyirf = zeros(nirf,rows(ZZ));

ssirf[1,.] = (RR*imp)';
yyirf[1,.] = ssirf[1,.]*ZZ';
t = 2;
do until t > nirf;
   ssirf[t,.] = (TT*ssirf[t-1,.]')';
   yyirf[t,.] = ssirf[t,.]*ZZ';
   t = t+1;
endo;

eofirf:

retp(yyirf);
endp;
