/*************************************************************
** filename:    parapmom.g
** description: Compute posterior mean and std and CI based on
**              output of posterior simulator
*************************************************************/

new;
closeall;
library user, pgraph;
cls;
outwidth 128;

mhrun  = "1";

nburn  = 10001;        
/* nburn  = 1;  */

/* Number of initial draws to be discarded 
** use nburn = 1 for draws from prior
** use the equivalent of 20 percent for draws from posterior */

hpdprob     = 0.90;
block_size  = 1E5;

#include dsge_mod.src;
#include dsge_para.src;
#include dsge_spec.g;

#include pathspec.g;

para  = pmsv(msel);
npara = rows(para);
{para,para_names} = psel(para,msel);

/******************************************************************
**         Load Parameter Draws from MH Output
******************************************************************/

lpath  = parapath $+ "\\mhrun" $+ mhrun;
opath  = parapath $+ "\\mhrun" $+ mhrun;

ldraw  = lpath $+ "\\pa"  ;
odraws = opath $+ "\\pas" ;

open fhdraw = ^ldraw for read;

drawrow = readr( fhdraw, nburn); 
drawdim = cols(drawrow);
drawrow = seekr( fhdraw, nburn);

/* Part 1: Compute the mean of x(i) and x^2(i)
*/
drawmean   = zeros(1,drawdim);
drawsqmean = zeros(1,drawdim);
drawcross  = zeros(drawdim,drawdim);

eofloop = 0;
ndraws  = 0;

do until eofloop; 

   drawblock = readr( fhdraw, block_size );

   drawmean  = drawmean    + sumc(drawblock)';
   drawsqmean = drawsqmean + sumc(drawblock^2)';
   drawcross  = drawcross  + drawblock'*drawblock;  

   ndraws = ndraws + rows(drawblock);

   locate 1,1;
   "Part 1";
   "Draws " ndraws;

   eofloop = eof(fhdraw);

endo;

drawmean   = real(drawmean/ndraws);
drawsqmean = real(drawsqmean/ndraws);
drawstdd   = real(sqrt(drawsqmean - (drawmean)^2));
drawcov    = real(drawcross/ndraws - drawmean'*drawmean);

/* Report Posterior Mean and stderror
**
*/
"Dataset     " dataselstr;
"MODEL       " lmodel;
"Subsample   " subT;
"PRIOR       " lprior;
"MH-Run      " mhrun;
"=============================================";
" " ;
   outmat = ("Parameter"  $~ "Mean" $~ "Stdd") $|
            ( (para_names') $~ ftocv( (drawmean' ~ drawstdd'),4,4) );
   $outmat;

 /* 
"Parameter | Mean  |  StdD |";
outmat = para_names'~ drawmean' ~ drawstdd' ;
let mask[1,3] = 0 1 1;
let fmt[3,3] =
   "-*.*s " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4;
d = printfm(outmat,mask,fmt);
*/

/* Part 2: HPD Interval
*/
cls;
drawrow     = seekr( fhdraw, nburn); 
drawci      = zeros(2,drawdim);

j = 1;
do until j > drawdim;

   /* Read only the j'th column
   */
   drawrow   = seekr( fhdraw, nburn); 
   drawblock = readr( fhdraw, 1);

   drawcol = drawblock[1,j];
   eofloop = 0;
   ndraws  = 1;
   
   do until eofloop; 

      drawblock = readr( fhdraw, block_size );
      drawcol   = drawcol | drawblock[.,j];
      ndraws    = ndraws + rows(drawblock);
      eofloop   = eof(fhdraw);
      locate 1,1;
      "Part 2";
      "ndraws" ndraws;
      "Column" j;
         
   endo;
   
   drawci[.,j] = hpdint(drawcol,hpdprob);
   j = j+1;

endo;
clear drawcol;

/* Report Posterior Mean and stderror
**
*/
cls;
"Dataset     " dataselstr;
"MODEL       " lmodel;
"Subsample   " subT;
"PRIOR       " lprior;
"MH-Run      " mhrun;
"Coverage    " hpdprob;  
"=============================================";
" " ;
   outmat = ("Parameter"  $~ "Mean" $~ "Stdd" $~ "CI(LOW)" $~ "CI(HIGH)")  
   $| ( (para_names') $~ ftocv( (drawmean' ~ drawstdd'~ drawci'),4,4));

   $outmat;

create fhdraws = ^odraws with DRAWSUM, drawdim, 8;
wr = writer(fhdraws, real(drawmean | drawstdd | drawcov | drawci) );
closeall fhdraws; 

end;

/**************************************************************/
/*                         Procedures                         */
/**************************************************************/

proc(1) = hpdint(draws,percent);
local     ndraws, drawdim, hpdband, nwidth, i, drawcoli,
          bup, minwidth, newwidth, j;

drawdim   = cols(draws);
ndraws    = rows(draws);
hpdband   = zeros(2,drawdim);
nwidth    = int(percent*ndraws);

i = 1;
do until i > drawdim;
   drawcoli = draws[.,i];
   /* sort response for period i, element 1 is max
   */
   drawcoli = rev(sortc(drawcoli,1));
   bup   = 1;
   minwidth  = drawcoli[1] - drawcoli[nwidth];
   j = 2;
   do until j > (ndraws-nwidth+1);
      newwidth = drawcoli[j] - drawcoli[j+nwidth-1];
      if newwidth < minwidth;
         bup = j;
         minwidth = newwidth;
      endif;
      j = j+1;
   endo;
   hpdband[2,i] = drawcoli[bup];
   hpdband[1,i] = drawcoli[bup+nwidth-1];
   i = i+1;
endo;
retp(hpdband);
endp;
