/* filename:    vm_acsamp.g
** description: The program computes impulse response 
**              functions for the VAR model
** created:     11/06/01
*/

new;
closeall;
library pgraph, user;
cls;
outwidth 256;

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
  __output = 1;
  _fcmptol = 1E-10;
  _verbose = 1;           /** 1 for verbose **/

#include vm_spec.g; 
#include pathspec.g;

/********************************************************************
* Load Parameter Draws from Posterior Sampler Output
********************************************************************/
mhrun  = "1";    /* "0" for draws from prior; "1" for draws from posterior */
nblock = 10000;  /* block size */
nuse   = 10;      /* use every nuse observation */
nobssim = 200;

lpath  = parapath $+ "\\mhrun" $+ mhrun;
opath  = resupath $+ "\\mhrun" $+ mhrun;
chkdir = chk_dir(opath);

lpara1  = lpath $+ "\\paravar1";
lpara2  = lpath $+ "\\paravar2";

open fhpara1 = ^lpara1 for read;
open fhpara2 = ^lpara2 for read;

/* Initialize Output files
*/
opath     = resupath $+ "\\mhrun" $+ mhrun;
chkdir    = chk_dir(opath);

ovaracsamp= opath $+ "\\varacsamp";

create fhvaracsamp = ^ovaracsamp with ACSAMP, 3, 8;

/* Initialization
*/ 
eofloop = 0;
loopct  = 1;
k = 2+nlags_*nvar_;

"Generating sample ACs...";

do until eofloop; 

   /** Read blocks of size nblock **/
   parasim1 = readr(fhpara1,nblock);
   parasim2 = readr(fhpara2,nblock);
   nread    = rows(parasim1);
   npara1   = cols(parasim1);

   /** use every nuse-th draw **/
   indseq  = seqa(1,1,nread);
   indseq  = indseq % nuse;
   indseq  = indseq .ne 0;

   parasim1 = delif(parasim1,indseq);
   parasim2 = delif(parasim2,indseq);
   nread    = rows(parasim1);

   if ismiss(parasim1) == 0;

   " ";
   "Loop      " loopct;
   "Blocksize " nread;
   eofloop = eof(fhpara1);

   acsampsim = zeros(nread,3);
   
   j = 1;
   do until j > rows(parasim1);
   
      if fmod(j,nread/10)==0;".";; pause(1e-20); endif;
   
      phi = reshape(parasim1[j,1:(nvar_-1)*k],nvar_-1,k)';
      sigma22 = parasim2[j,.];

      acsampsim[j,.] = varacsamp(Phi,Sigma22,nobssim);

      j = j+1;
  endo;
      
  /* Write Output
  */
  wr = writer(fhvaracsamp,acsampsim);

  endif;

  loopct = loopct + 1;

endo;

closeall;

"";
"DONE";
end;

/********************************************************************
**   Procedure to compute Impulse Response Function
********************************************************************/
proc (1) = varacsamp(Phi,Sigma22,nobssim);
local nvar, nlags, phi_sim, phi_target, imp, YYsim, XXpred, j,
      corrYY, ac_sim;

nvar  = cols(Phi)+1;
nlags = (rows(Phi)-2)/nvar;

/* remove intercept and target-diff
*/
phi_sim = phi[1:nvar*nlags,.];

/* add target inflation eq
*/
if nlags == 1;
   phi_target = (0|0|0|0|1);
else;
   phi_target = (  (0|0|0|0|1) 
                 | zeros( nvar*(nlags-1), 1) );
endif;

phi_sim = phi_sim ~ phi_target;

/* create impact for inflation target shock
*/
imp = sqrt(sigma22)*( phi[nvar*nlags+1,.]~1 );

/* compute impulse responses
*/
YYsim  = zeros(nobssim,nvar);
XXpred = zeros(1,nvar*nlags);

j = 1;
do until j > nirf;

   YYsim[j,.] = XXpred*phi_sim + imp*rndn(1,1);

   if nlags > 1;
      XXpred[nvar+1:nvar*nlags] = XXpred[1:(nlags-1)*nvar];
   endif;
   XXpred[1:nvar] = YYsim[j,.];
   
   j = j+1;
   
endo;

corrYY = corrx(YYsim[.,3]~YYsim[.,4]/100);

ac_sim = stdc(YYsim[.,3])
       ~ stdc(YYsim[.,4]/100)
       ~ corrYY[1,2];

retp(ac_sim);
endp;

