/******************************************************************/
/* filename :       vm_postdraws.g                                */
/* Description:     Estimating the paramaters of a BVAR           */
/*                                                                */
/*                  Cristina and Leonardo                         */
/* last updated  :  06:47 PM  29-Aug-2007                         */
/******************************************************************/

new;
closeall; 
library pgraph;
cls;

#include vm_spec.g; 
#include pathspec.g;

_verbose = 1;    /* set it equal to 1 for detailed printing of code outputs */
mhrun    = "1";  /* posterior draws  */

/******************************************************** 
* Import data series                                    *
********************************************************/

#include vm_loaddata.g;
nobs    =   rows(YY)-T0; 

/******************************************************** 
* Initialize directories, etc...                        *
********************************************************/

parapath = parapath $+ "\\" $+ "\mhrun" $+ mhrun;
chkdir = chk_dir(parapath);

/** output file name 
**/
oparavar1 = parapath $+ "\\paravar1.dat";
oparavar2 = parapath $+ "\\paravar2.dat";

/** create output file 
**/
numcols1 = 2*(nvar_-1) + nlags_*nvar_*(nvar_-1)+ 0.5*(nvar_-1)*nvar_; 
create fhparavar1 = ^oparavar1 with PARA, numcols1, 8;  

numcols2 = 1; 
create fhparavar2 = ^oparavar2 with PARA, numcols2, 8;  

/** check if you have run vm_dummy.g first 
**/
ldumobs1 = priopath $+ "\\dumobs1" $+ ".dat";
if not(exist(ldumobs1));
    "";
    "Please run vm_dummy.g first";
    "";
    end;
endif;

ldumobs2 = priopath $+ "\\dumobs2" $+ ".dat";
if not(exist(ldumobs2));
    "";
    "Please run vm_dummy.g first";
    "";
    end;
endif;

/******************************************************** 
** open output generated by vm_dummy.g 
** dummy  YY1dum XXdum YY2dum
** actual YY1act XXact YY2act
********************************************************/

open fhdumobs1 = ^ldumobs1;
YY1XX = readr(fhdumobs1,1);  
do until eof(fhdumobs1);
   YY1XX = YY1XX | readr(fhdumobs1,10);
endo;
YY1 = YY1XX[.,1:nvar_-1];
XX  = YY1XX[.,nvar_:cols(YY1XX)];
n_dummy1 = rows(YY1)-nobs;


open fhdumobs2 = ^ldumobs2;
YY2 = readr(fhdumobs2,1);  
do until eof(fhdumobs2);
   YY2 = YY2 | readr(fhdumobs2,10);
endo;
n_dummy2 = rows(YY2)-nobs;

/* Defining constants 
*/ 
k       = cols(XX);      /* number of parameters per equation  */

/* OLS ESTIMATORS
*/
phi_tilde     =   (XX'*YY1)/(XX'*XX) ;
sigma11_tilde =   (1/(nobs+n_dummy1))*((YY1'*YY1)-(XX'*YY1)'( (XX'*YY1)/(XX'*XX) ));
sigma22_tilde =   (1/(nobs+n_dummy1))*(YY2'*YY2);
rmses         =   (1/nobs)*(YY1 - XX*Phi_tilde)'*(YY1 - XX*Phi_tilde);
rmses         =   sqrt(diag(rmses));
rmses;


/******************************************************** 
** Direct Sampling
********************************************************/

nblocks     =    10;
nsim        =    1000;
nsim_total  =    nblocks*nsim;

phi_mean     =   zeros(1,(nvar_-1)*k);
sigma11_mean =   zeros(1,0.5*nvar_*(nvar_-1));
sigma22_mean =   zeros(1,1);

iblock      =    1;

do until iblock > nblocks;

    if _verbose;
        "";
        "----------------------------------------";
        "Block" iblock "of" nblocks;
        pause(1e-20);
    endif;

    /** set start time **/
    tstart = date;    

    phi     = zeros(nsim,cols(phi_tilde)*rows(phi_tilde));    
    sigma11 = zeros(nsim,0.5*rows(sigma11_tilde)*(rows(sigma11_tilde)+1));
    sigma22 = zeros(nsim,1);

    phi_new = vec(phi_tilde);
    
    j=1;
   
    do until j > nsim;

        /* Drawing from Sigma ~ IW((nobs+n_dummy)*sigma_tilde,v)*/
        phi_mat = reshape(phi_new,nvar_-1,k)';
        sigma11_tilde = (1/(nobs+n_dummy1))*(YY1-XX*phi_mat)'*(YY1-XX*phi_mat);
        sigma11_new = iwish(nobs+n_dummy1,(nobs+n_dummy1)*sigma11_tilde); 
        sigma22_new = iwish(nobs+n_dummy2-1,(nobs+n_dummy2)*sigma22_tilde);
               
        /* Drawing from Phi|Sigma ~ N(phi_tilde, sigma_new.*.inv(XXstar'*XXstar + XX'*XX))
        ** enforce non-explosiveness */
        valid = 0;
        do until valid == 1;
           phi_new = mrndn(vec(phi_tilde),sigma11_new.*.inv(XX'*XX)); 
           phi_mat = reshape(phi_new',nvar_-1,k)';
           /* eliminate last two rows */
           phi_mat = phi_mat[1:nvar_*nlags_,.];
           /* add inflation target equation */
           if nlags_ == 1;
              phi_target = (0|0|0|0|1);
           else;
              phi_target = (  (0|0|0|0|1) 
                             | zeros( nvar_*(nlags_-1), 1) ); 
           endif;
           phi_mat = phi_mat ~ phi_target;
           /* create companion form */
           phi_mat = phi_mat';
           if nlags_ > 1;
              phi_mat = phi_mat |
                        ( eye( (nlags_-1)*nvar_ ) ~ zeros( (nlags_-1)*nvar_,nvar_) );
           endif;
           valid = (maxc(abs(eig(phi_mat))) .le 1);
        endo;
 
        /* Store draws */
        sigma11[j,.]  = vech(sigma11_new)';  
        sigma22[j,.]  = vech(sigma22_new)';  
        phi[j,.]      = phi_new';

        j=j+1;
        if _verbose;
            if fmod(j,nsim/10)==0;".";; pause(1e-20); endif;
        endif;

    endo;

    /** set finish time **/
    tfin = date;

    if _verbose;
        "";
        "Elapsed Time"; etstr(ethsec(tstart,tfin));
        "";
    endif;  /** verbose **/
    
    phi_mean     = phi_mean     + sumc(phi)';
    sigma11_mean = sigma11_mean + sumc(sigma11)';
    sigma22_mean = sigma22_mean + sumc(sigma22)';

    wr = writer(fhparavar1,(phi~sigma11));
    wr = writer(fhparavar2,sigma22);

    iblock=iblock+1;

endo;

closeall;

/* getting the estimates for parameters and var-cov matrix of residuals */
phi_mean     = phi_mean/nsim_total;
sigma11_mean = sigma11_mean/nsim_total;
sigma22_mean = sigma22_mean/nsim_total;

PHI_hat      = reshape(phi_mean,nvar_-1,k)';
SIGMA11_hat  = xpnd(sigma11_mean');
SIGMA22_hat  = xpnd(sigma22_mean');

if _verbose;

/** start log file **/
outstr = resupath $+ "\\para.log"; 
output file = ^outstr reset;

    "";
    "===================================";
    "Model            " $lmodel;
    "Data set         " $dataselstr;
    "Prior            " $lprior;
    "Subsample        " $subTstr;
    "";
    "===================================";
    "";
    "";
    
   "PHI =";
    PHI_hat;
    "";

    "SIGMA11 =";
    SIGMA11_hat;
    "";

    "SIGMA22 =";
    SIGMA22_hat;
    "";

    /** close log file **/
    output off;

endif;

end;
    
/***************************************************************************************************************************************************/


/* This procedure samples from an inverted-Wishart distribution                                                     */

proc(1) = iwish(v,S);  
/* where v is the df and S is the location parameter defined as the sumof squared residuals ie (Y-X*beta)'*(Y-X*beta) */

local nvar, C, Z, IW, W;

nvar    = cols(S);
C       = cholpsd(inv(S))';
Z       = C*rndn(nvar,v);
W      = Z*Z';
IW       = inv(W);
retp(IW);
endp;


/***************************************************************************************************************************************************/



/* This procedure samples from a multivariate normal distribution with mean mu and var-cov matrix Sigma             *
/ NOTE that: this command can also be used for sampling from a non standard univariate normal                       */

proc(1) = mrndn(mu,Sigma);

local nvar, C, Z, i;

nvar    = cols(Sigma);
C       = cholpsd(Sigma);
Z       = mu+C*rndn(nvar,1);
retp(Z);
endp;

/***************************************************************************************************************************************************/

