/***************************************************************/
/*                                                             */
/*     VAR analysis: compute marginal data density             */
/*                                                             */
/***************************************************************/

new;
closeall; 
library pgraph;
cls;

#include vm_spec.g; 
#include pathspec.g;

/******************************************************** 
* Import data series to determine length of sample      *
********************************************************/

#include vm_loaddata.g;
/** number of actual observations
**/  
nobs    =   rows(YY)-T0;  

/******************************************************** 
* Load Y and XX matrices for dummy and actual observations  
********************************************************/

/** check if you have run vm_dummy.g first 
**/
ldumobs1 = priopath $+ "\\dumobs1" $+ ".dat";
if not(exist(ldumobs1));
    "";
    "Please run vm_dummy.g first";
    "";
    end;
endif;

ldumobs2 = priopath $+ "\\dumobs2" $+ ".dat";
if not(exist(ldumobs2));
    "";
    "Please run vm_dummy.g first";
    "";
    end;
endif;

/******************************************************** 
** open output generated by vm_dummy.g 
** dummy  YY1dum XXdum YY2dum
** actual YY1act XXact YY2act
********************************************************/

open fhdumobs1 = ^ldumobs1;
YY1XX = readr(fhdumobs1,1);  
do until eof(fhdumobs1);
   YY1XX = YY1XX | readr(fhdumobs1,10);
endo;
YY1 = YY1XX[.,1:nvar_-1];
XX  = YY1XX[.,nvar_:cols(YY1XX)];
n_total1 = rows(YY1);
n_dummy1 = n_total1-nobs;
YY1dum = YY1[1:n_dummy1,.];
XXdum  = XX[1:n_dummy1,.];
nv1 = cols(YY1);

open fhdumobs2 = ^ldumobs2;
YY2 = readr(fhdumobs2,1);  
do until eof(fhdumobs2);
   YY2 = YY2 | readr(fhdumobs2,10);
endo;
n_total2 = rows(YY2);
n_dummy2 = n_total2-nobs;
YY2dum = YY2[1:n_dummy2,.];
nv2 = cols(YY2);

k       = cols(XX); 

/** Compute the log marginal data density for the VAR model 
**/ 

/* Part 1 */

Phi0   = (XXdum'*YY1dum)/(XXdum'*XXdum);
S0     = YY1dum'*YY1dum-(XXdum'*YY1dum)'( (XXdum'*YY1dum)/(XXdum'*XXdum) ); 

Phi1   = (XX'*YY1)/(XX'*XX); 
S1     = YY1'*YY1-(XX'*YY1)'( (XX'*YY1)/(XX'*XX) );

/* compute constants for integrals
*/

i=1;  
gam0=0; 
gam1=0; 

do until i > nv1; 
   gam0=gam0+lngam(0.5*(n_dummy1-k+1-i)); 
   gam1=gam1+lngam(0.5*(n_total1-k+1-i)); 
   i=i+1; 
endo; 

/* dummy observation
*/
lnpY0 = -nv1*(n_dummy1-k)*0.5*ln(pi)-(nv1/2)*ln(abs(det(XXdum'XXdum)))
        -(n_dummy1-k)*0.5*ln(abs(det(S0)))+nv1*(nv1-1)*0.25*ln(pi)+gam0;

/* dummy and actual observations
*/   
lnpY1 = -nv1*(n_total1-k)*0.5*ln(pi)-(nv1/2)*ln(abs(det(XX'XX)))
        -(n_total1-k)*0.5*ln(abs(det(S1)))+nv1*(nv1-1)*0.25*ln(pi)+gam1;  

lnpYY1 = lnpY1-lnpY0; 

/**********************************************************************/

/* Part 2 */

S0     = YY2dum'*YY2dum; 
S1     = YY2'*YY2;

/* compute constants for integrals
*/

i=1;  
gam0=0; 
gam1=0; 

do until i > nv2; 
   gam0=gam0+lngam(0.5*(n_dummy2+1-i)); 
   gam1=gam1+lngam(0.5*(n_total2+1-i)); 
   i=i+1; 
endo; 

/* dummy observation
*/
lnpY0 = -nv2*(n_dummy2)*0.5*ln(pi)
        -(n_dummy2)*0.5*ln(abs(det(S0)))+nv2*(nv2-1)*0.25*ln(pi)+gam0;

/* dummy and actual observations
*/   
lnpY1 = -nv2*(n_total2)*0.5*ln(pi)
        -(n_total2)*0.5*ln(abs(det(S1)))+nv2*(nv2-1)*0.25*ln(pi)+gam1;  

lnpYY2 = lnpY1-lnpY0; 

/** start log file **/
outstr = resupath $+ "\\mdd.log"; 
output file = ^outstr reset;

"";
    "===================================";
    "Model            " $lmodel;
    "Data set         " $dataselstr;
    "Prior            " $lprior;
    "Subsample        " $subTstr;
    "";
    "===================================";
    "";
    
    "";
    "Log-marginal Density (Part 1) =" lnpYY1; 
    "";
    "Log-marginal Density (Part 2) =" lnpYY2; 
    "";
    "Log-marginal Density (Combined) =" lnpYY1+lnpYY2; 




"";
